package projekt.enigma.model;

/**
 * <p>
 * Klasse Steckbrett
 * <br>
 * Das Steckbrett diente dazu, Buchstaben paarweise zu vertauschen noch bevor diese durch die Walzen codiert werden.
 * Die Klasse Steckbrett nimmt bis zu 10 paarweise zu vertauschende Buchstabenpaare entgegen, prueft diese auf
 * Zulaessigkeit und tauscht diese dann paarweise aus.
 */
public class Steckbrett {

    /**
     * orginalBuchstaben: String : der jeweils den ersten zu vertauschenden Buchstaben enthaelt.
     */
    private String originalBuchstaben;

    /**
     * getauschteBuchstaben: String : der jeweils den zweiten zu vertauschenden Buchstaben enthaelt.
     */
    private String getauschteBuchstaben;

    /**
     * Konstruktor Steckbrett
     * <br>
     * Erzeugt zwei Strings und initialisiert diese.
     */
    public Steckbrett() {
        originalBuchstaben = "";
        getauschteBuchstaben = "";
    }

    /**
     * Funktion setzeVertauschung
     * <br>
     * Der Funktion setzeVertauschung werden 2 Buchstaben (buchstabe1 und buchstabe2) uebergeben.
     * Zuerst wird ueberprueft ob einer der beiden Buchstaben bereits enthalten ist, was unzulaessig waere.
     * <br>
     * Anschliessend wird:
     * <p>
     * buchstabe1 dem String orginalBuchstaben angehaengt.
     * buchstabe2 dem String getauschteBuchstaben angehaengt.
     * </p>
     *
     * @param buchstabe1 : char : Gibt den ersten Buchstaben an, der getauscht werden soll.
     * @param buchstabe2 : char : gibt den zweiten Buchstaben an, mit dem der erste getauscht werden soll.
     * @return boolean : Wenn die Vertauschung zulaessig ist, true, ansonsten false.
     */
    public boolean setzeVertauschung(char buchstabe1, char buchstabe2) {
        if ((!ueberpruefeVertauschungen(buchstabe1)) && (!ueberpruefeVertauschungen(buchstabe2))
                && (originalBuchstaben.length() <= 20)) {
            originalBuchstaben += buchstabe1;
            getauschteBuchstaben += buchstabe2;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Funktion setzeVertauschung mit Angabe eines Ports von 1 - 10
     * <br>
     * Der Funktion setzeVertauschung werden 2 Buchstaben (buchstabe1 und buchstabe2) sowie der Port uebergeben.
     * Zuerst wird ueberprueft ob einer der beiden Buchstaben bereits enthalten ist, was unzulaessig waere.
     * <br>
     * Anschliessend wird:
     * <p>
     * buchstabe1 in den String orginalBuchstaben auf die Position (steckbrettPort) geschrieben.
     * buchstabe2 in den String getauschteBuchstaben auf die Position (steckbrettPort) geschrieben.
     * </p>
     *
     * @param buchstabe1 : char: Gibt den ersten Buchstaben an, der getauscht werden soll.
     * @param buchstabe2 : char: gibt den zweiten Buchstaben an, mit dem der erste getauscht werden soll.
     * @return boolean : Wenn die Vertauschung zulaessig ist, true, ansonsten false.
     */
    public boolean setzeVertauschung(int steckbrettPort, char buchstabe1, char buchstabe2) {
        char[] original;
        char[] getauscht;

        new Fehler().debug("setzeVertauschung(int " + steckbrettPort + ", char |" + buchstabe1 + "|, char |" + buchstabe2 + "|)", false);

        if ((!ueberpruefeVertauschungen(buchstabe1) && !ueberpruefeVertauschungen(buchstabe2)) ||
                (buchstabe1 == ' ' && buchstabe2 == ' ')) {

            new Fehler().debug("-> Original:\t" + originalBuchstaben, false);
            new Fehler().debug("-> Getauscht:\t" + getauschteBuchstaben, false);
            original = originalBuchstaben.toCharArray();
            getauscht = getauschteBuchstaben.toCharArray();

            original[steckbrettPort] = buchstabe1;
            getauscht[steckbrettPort] = buchstabe2;

            originalBuchstaben = String.valueOf(original);
            getauschteBuchstaben = String.valueOf(getauscht);
            new Fehler().debug("<- Original:\t" + originalBuchstaben, false);
            new Fehler().debug("<- Getauscht:\t" + getauschteBuchstaben, false);

            new Fehler().debug("true\tOS: |" + originalBuchstaben + "| GS: |" + getauschteBuchstaben + "|", false);
            return true;
        } else {
            new Fehler().debug("false\tOS: |" + originalBuchstaben + "| GS: |" + getauschteBuchstaben + "|", false);
            return false;
        }
    }

    /**
     * Funktion ueberpruefeVertauschungen
     * <br>
     * Die Funktion ueberpruefeVertauschungen prueft, ob der uebergebene Buchstabe bereits in den beiden Strings
     * orginalBuchstaben und getauschteBuchstaben vorhanden ist.
     * Eine Buchstabenvertauschung ist zulaessig, wenn keine Dopplungen der zu vertauschenden Buchstaben auftritt
     * (ein Buchstabe nicht mehrmals vertauscht wird).
     *
     * @param buchstabe : Character: zu pruefender Buchstabe.
     * @return result : boolean : Wenn false, dann darf der Buchstabe genutzt werden. Wenn er schon vorhanden ist,
     * wird true zurueckgegeben.
     */
    public boolean ueberpruefeVertauschungen(Character buchstabe) {
        boolean result = false;

        if (originalBuchstaben.contains(String.valueOf(buchstabe)) ||
                getauschteBuchstaben.contains(String.valueOf(buchstabe))) {
            result = true;
        }

        return result;
    }

    /**
     * Funktion codiere
     * <br>
     * Die Funktion codiere tauscht den uebergebenen Character buchstabe mit dem zu tauschenden Character aus.
     *
     * @param buchstabe : Character : Buchstabe der codiert werden soll.
     * @return buchstabe : Character : Codierter Buchstabe.
     */
    public char codiere(Character buchstabe) {
        if (originalBuchstaben.contains(String.valueOf(buchstabe))) {
            return getauschteBuchstaben.charAt(originalBuchstaben.indexOf(buchstabe));
        } else if (getauschteBuchstaben.contains(String.valueOf(buchstabe))) {
            return originalBuchstaben.charAt(getauschteBuchstaben.indexOf(buchstabe));
        } else {
            return buchstabe;
        }
    }

    /**
     * Tauscht die Buchstaben im Alphabet aus und gibt ein modifiziertes Alphabet zurueck.
     *
     * @param port : int : Platz der getauschten Buchstaben.
     * @return alphabet : String : Alphabet mit getauschtem Buchstabenpaar.
     */
    public String fetchVerfuegbareBuchstaben(int port) {
        String alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

        for (char c : this.originalBuchstaben.toCharArray()) {
            alphabet = alphabet.replace(String.valueOf(c), "");
        }

        for (char c : this.getauschteBuchstaben.toCharArray()) {
            alphabet = alphabet.replace(String.valueOf(c), "");
        }

        alphabet += originalBuchstaben.charAt(port);
        alphabet += getauschteBuchstaben.charAt(port);

        return alphabet;
    }
}