package projekt.enigma.view;

import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ToggleButton;
import javafx.scene.layout.GridPane;
import projekt.enigma.model.Codierer;

import java.util.Arrays;

/**
 * Die Klasse ChkBoxGripPane erweitert das GridPane Objekt.
 * Wir erstellen ein GridPane in welches Checkboxen gelegt werden.
 * Die Anzahl der Checkboxen wird durch die verfuegbarenBuchstaben festgelegt.
 * Für jeden Buchstaben in diesem char Array, wird eine Checkbox erstellt und auf das Gridpane gelegt.
 */
public class ChkBoxGridPane extends GridPane {

    // Der globale Checkbox Array
    private CheckBox[] checkbox;
    // Die Buchstaben die für das jeweilige Pane selectiert werden sollen
    private String checkedBuchstaben;
    // Die Anzahl der möglichen Checkboxen
    private int charSize;
    // Das Codierer Objekt. Dieses wird benötigt um auf die Funktionen für das Steckbrett zuzugreifen
    private Codierer codierer;
    // Der Steckbrett Port für welchen das Pane und die Checkboxen erstellt werden sollen
    private int port;
    // Die verfügbaren Buchstaben für den jeweiligen Port
    private char[] verfuegbareBuchstaben;
    // Das Button Objekt des Ports über den dieses Pane gesteuert wird
    private ToggleButton tBtnPort;

    /**
     * Hier werfen wir die Checkboxen auf das Gridpane und weißen ihnen die Propertys zu.
     * Außerdem wird hier auch der Listener generiert womit wir prüfen ob die maximale Auswahlmöglichkeit
     * erreicht ist.
     * Es können maximal zwei Checkboxen ausgewählt werden. Wenn zwei erreicht sind, werden die restlichen Checkboxen
     * deaktiviert. Wenn nur noch eine oder keine Box ausgewählt ist, werden die restlichen wieder aktiviert.
     * Bei null und zwei Boxen, werden die Buchstaben in das Steckbrett geschrieben.
     * Andere Auswahlmöglchkeiten stehen nicht zur Verfügung, da ein Kabel nur zwei Enden hat in der Regel.
     *
     * @param codierer : Codierer Objekt
     * @param port     : int
     * @param tBtnPort : ToogleButton
     */
    public ChkBoxGridPane(Codierer codierer, int port, ToggleButton tBtnPort) {
        this.codierer = codierer;
        this.port = port;
        this.tBtnPort = tBtnPort;
        this.verfuegbareBuchstaben = this.codierer.fetchVerfuegbareBuchstaben(this.port).toCharArray();
        Arrays.sort(this.verfuegbareBuchstaben);
        charSize = this.verfuegbareBuchstaben.length;
        this.checkedBuchstaben = tBtnPort.getText();
        initializeComponents();
    }

    /**
     * Es wird ein GridPane erzeugt, das zwei Spalten und abhängig von der Länge des
     * Arrays (Character[] alphabet) Reihen hat. Das Label der CheckBoxen soll
     * abhängig von deren Index links bzw. recht erzeugt werden
     */
    private void initializeComponents() {
        checkbox = new CheckBox[charSize];

        this.setAlignment(Pos.CENTER);

        // Für jeden verfügbaren Buchstaben einen Durchlauf
        for (int i = 0; i < this.verfuegbareBuchstaben.length; i++) {
            if (this.verfuegbareBuchstaben[i] != 20 && this.verfuegbareBuchstaben[i] != ' ') {
                checkbox[i] = new CheckBox(this.verfuegbareBuchstaben[i] + "");
                checkbox[i].setPadding(new Insets(1, 1, 1, 1));

                if (!checkedBuchstaben.equals("")) {
                    if (!checkedBuchstaben.contains(String.valueOf(this.verfuegbareBuchstaben[i]))) {
                        checkbox[i].setDisable(true);
                    } else {
                        checkbox[i].setSelected(true);
                    }
                } else {
                    checkbox[i].setSelected(false);
                    checkbox[i].setDisable(false);
                }

                // Der Listener für die jeweilige Checkbox
                checkbox[i].selectedProperty().addListener((observable, oldValue, newValue) -> {
                    String portSelection = "";
                    int zaehler = 0;

                    // Die selektierten Chechkboxen zählen
                    for (CheckBox cb : checkbox) {
                        if (cb != null && cb.isSelected()) {
                            zaehler++;
                        }
                    }
                    // Wenn nur eine oder keine Checkbox ausgewählt ist
                    if (zaehler == 1 || zaehler == 0) {
                        for (CheckBox cb : checkbox) {
                            if (cb != null && cb.isDisabled()) {
                                cb.setDisable(false);
                            }
                        }
                    }
                    // Es ist keine Checkbox ausgewählt
                    if (zaehler == 0) {
                        // Daten ins Steckbrett schreiben
                        this.codierer.setzeSteckbrett(this.port, "");
                        // Und den Button neu beschreiben
                        tBtnPort.setText(portSelection);
                    }
                    // Das Maximum an selektierten Checkboxen (2) ist erreicht
                    if (zaehler == 2) {
                        for (CheckBox cb : checkbox) {
                            if (cb != null && !cb.isSelected()) {
                                cb.setDisable(true);
                            } else {
                                if (cb != null) {
                                    portSelection += cb.getText();
                                }
                            }
                        }
                        // Daten ins Steckbrett schreiben
                        this.codierer.setzeSteckbrett(this.port, "");
                        this.codierer.setzeSteckbrett(this.port, portSelection);
                        // Und den Button neu beschreiben
                        tBtnPort.setText(portSelection);
                    }
                });

                // Eine neue Row im Gridpane anlegen für den nächsten Button
                this.addRow(i + 1);
                this.add(checkbox[i], 2, i + 1, 1, 1);
            }
        }
        // Das Pane anzeigen
        this.setVisible(true);
    }

    /**
     * Gibt den Anzahl der zur Verfügung stehenden Buchstaben zurück.
     * Anhang dieses Wertes, wird die Anzahl der Checkboxen festgelegt.
     *
     * @return int : Anzahl der möglcihen chars
     */
    public int getCharSize() {
        return charSize;
    }
}

